package servers;

import java.net.InetSocketAddress;
import java.nio.ByteBuffer;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

import main.Main;

// websocket to interpret user's commands
public class DataWebSocket extends WebSocketServer {
    
    private Main main;

    public DataWebSocket(InetSocketAddress address, Main m) {
    	super(address);
    	main=m;
    }
    

    @Override
	public void onOpen(WebSocket conn, ClientHandshake handshake) {
		System.out.println("new connection to " + conn.getRemoteSocketAddress());
	}

	@Override
	public void onClose(WebSocket conn, int code, String reason, boolean remote) {
		System.out.println("closed " + conn.getRemoteSocketAddress() + " with exit code " + code + " additional info: " + reason);
	}

	@Override
	public void onMessage(WebSocket conn, String message) {		// get and interpret commands from web interface
		System.out.println("received message from "	+ conn.getRemoteSocketAddress() + ": " + message);
		
		String[] elements=message.split(" ");
		
		// robot control commands
		if (elements.length>=2 && elements[0].equals("robot")){
			main.robotCommand(elements[1]);
		}
		
		// system control commands
		if (elements.length>=2 && elements[0].equals("system")){
			main.systemCommand(elements[1]);
		}
		
	}

	@Override
	public void onMessage( WebSocket conn, ByteBuffer message ) {
		System.out.println("received ByteBuffer from "	+ conn.getRemoteSocketAddress());
	}

	@Override
	public void onError(WebSocket conn, Exception ex) {
		System.err.println("an error occurred on connection " + conn.getRemoteSocketAddress()  + ":" + ex);
	}
	
	@Override
	public void onStart() {
		System.out.println("server started successfully");
	}
}